# -*- coding: utf-8 -*-
import os
import zipfile
import datetime
import logging
from .models import Backup
from django.conf import settings
from six.moves import configparser
from YaDiskClient.YaDiskClient import YaDisk
from django.db import connection


class ZipUtilities(object):
    def zip(self, path, zip_file):
        if os.path.isfile(path):
            zip_file.write(path)
            print('File added: ' + str(path))
        else:
            self.add_folder_to_zip(zip_file, path)

    def add_folder_to_zip(self, zip_file, path):
        for file in os.listdir(path):
            full_path = os.path.join(path, file)
            if os.path.isfile(full_path):
                print('File added: ' + str(full_path))
                zip_file.write(full_path)
            elif os.path.isdir(full_path):
                print('Entering folder: ' + str(full_path))
                self.add_folder_to_zip(zip_file, full_path)


class DatabaseUtilities(object):

    def parse_mysql_cnf(self, dbinfo):
        """
        Attempt to parse mysql database config file for connection settings.
        Ideally we would hook into django's code to do this, but read_default_file is handled by the mysql C libs
        so we have to emulate the behaviour
        Settings that are missing will return ''
        returns (user, password, database_name, database_host, database_port)
        """
        read_default_file = dbinfo.get('OPTIONS', {}).get('read_default_file')
        if read_default_file:
            config = configparser.RawConfigParser({
                'user': '',
                'password': '',
                'database': '',
                'host': '',
                'port': '',
                'socket': '',
            })
            config.read(os.path.expanduser(read_default_file))
            try:
                user = config.get('client', 'user')
                password = config.get('client', 'password')
                database_name = config.get('client', 'database')
                database_host = config.get('client', 'host')
                database_port = config.get('client', 'port')
                socket = config.get('client', 'socket')

                if database_host == 'localhost' and socket:
                    # mysql actually uses a socket if host is localhost
                    database_host = socket

                return user, password, database_name, database_host, database_port

            except configparser.NoSectionError:
                pass

        return '', '', '', '', ''

    def dump(self, output_directory, zip_file):
        """
        Create a default database dump and add it to archive
         TODO: add support for PostgreSQL and SqlLight
        """
        engine = connection.vendor
        if engine == 'mysql':
            (user, password, database_name, database_host, database_port) = self.parse_mysql_cnf(settings.DATABASES)
            user = settings.DATABASES.get('USER') or user
            password = settings.DATABASES.get('PASSWORD') or password
            database_name = settings.DATABASES.get('NAME') or database_name
            sql_file_name = "%s.sql" % database_name

            cmd = "%(mysqldump)s -u %(user)s --password=%(password)s %(database)s > %(log_dir)s%(file)s" % {
                'mysqldump': 'mysqldump',
                'user': user,
                'password': password,
                'database': database_name,
                'log_dir': output_directory,
                'file': sql_file_name
            }
            logging.debug("Backing up with command %s " % cmd)
            os.system(cmd)
            print(cmd)

            full_sql_path = os.path.join(output_directory, sql_file_name)

            if os.path.exists(full_sql_path):
                utilities = ZipUtilities()
                utilities.zip(full_sql_path, zip_file)
                os.remove(full_sql_path)


class YaBackup(object):
    """
    Main object to work with backups
    TODO: backup Media folder as a single option    
    """
    DATETIME_FORMAT = settings.YABACKUP_SETTINGS['DATE_TIME_FORMAT']
    disk = YaDisk(settings.YABACKUP_SETTINGS['YADISK_LOGIN'], settings.YABACKUP_SETTINGS['YADISK_PASSWORD'])

    def run_backups(self, queryset=None, manual=False):
        """        
        Main function to run backups (manual or ron based)
        """
        if manual:
            backup_list = queryset
        else:
            backup_list = Backup.objects.all()

        for backup in backup_list:
            # Add a slash to output directory path if it's not there and create this folder
            output_directory = os.path.join(backup.output_directory, '')
            if not os.path.exists(output_directory):
                os.mkdir(output_directory)
                print('Root folder created: ', output_directory)

            backup_date = datetime.datetime.now().strftime(self.DATETIME_FORMAT + '_')
            file_name = output_directory + backup_date + backup.file_name
            paths_list = backup.paths.all()
            zip_file = zipfile.ZipFile(file_name, 'w', compression=zipfile.ZIP_DEFLATED)

            for path in paths_list:
                if os.path.exists(path.path):
                    utilities = ZipUtilities()
                    utilities.zip(path.path, zip_file)
                else:
                    print("File or folder doesn't exist: ", path)

            if backup.mysqldump:
                utilities = DatabaseUtilities()
                utilities.dump(output_directory, zip_file)

            zip_file.close()

            if backup.upload:
                self.disk.upload(file_name, settings.YABACKUP_SETTINGS['YADISK_BACKUP_ROOT'] + backup_date + backup.file_name)
                print(file_name, ' successfully uploaded')

                if backup.delete_after_upload:
                    os.remove(file_name)
                    print(file_name, ' was removed')

# https://github.com/pypa/twine


# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _


class Backup(models.Model):
    title = models.CharField(blank=True, max_length=128, verbose_name=_('Title'), help_text=_('Just a title'))
    file_name = models.CharField(max_length=64, verbose_name=_('File name'), help_text=_('Date and time will be added to the file name automagically. Example: backup.zip'))
    output_directory = models.CharField(max_length=256, verbose_name=_('Output directory'), help_text=_('Where to store files localy? Example: /home/user/backups/'))
    upload = models.BooleanField(default=False, verbose_name=_('Upload to Yandex.Disk'))
    delete_after_upload = models.BooleanField(default=False, verbose_name=_('Delete after upload'))
    mysqldump = models.BooleanField(default=False, verbose_name=_('MySQL dump'), help_text=_("Create a backup of the projects's database"))
    pub_date = models.DateTimeField(blank=True, auto_now_add=True, verbose_name=_('Published'))
    description = models.TextField(blank=True, verbose_name=_('Description'))

    class Meta:
        verbose_name = _('Backup')
        verbose_name_plural = _('Backups')

    def __str__(self):
        return self.title


class Path(models.Model):
    backup = models.ForeignKey('Backup', related_name='paths', on_delete=models.CASCADE)
    path = models.CharField(max_length=256, verbose_name=_('Path'), help_text=_('Absolute path to file or folder'))

    class Meta:
        verbose_name = _('file or folder')
        verbose_name_plural = _('Files and folders')

    def __str__(self):
        return self.path





# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-21 15:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Backup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, help_text='Just a title', max_length=128, verbose_name='Title')),
                ('file_name', models.CharField(help_text='Date and time will be added to the file name automagically. Example: backup.zip', max_length=64, verbose_name='File name')),
                ('output_directory', models.CharField(help_text='Where to store files localy? Example: /home/user/backups/', max_length=256, verbose_name='Output directory')),
                ('upload', models.BooleanField(default=False, verbose_name='Upload to Yandex.Disk')),
                ('delete_after_upload', models.BooleanField(default=False, verbose_name='Delete after upload')),
                ('mysqldump', models.BooleanField(default=False, help_text="Create a backup of the projects's database", verbose_name='MySQL dump')),
                ('pub_date', models.DateTimeField(auto_now_add=True, verbose_name='Published')),
                ('description', models.TextField(blank=True, verbose_name='Description')),
            ],
            options={
                'verbose_name_plural': 'Backups',
                'verbose_name': 'Backup',
            },
        ),
        migrations.CreateModel(
            name='Path',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(help_text='Absolute path to file or folder', max_length=256, verbose_name='Path')),
                ('backup', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='paths', to='YaBackup.Backup')),
            ],
            options={
                'verbose_name_plural': 'Files and folders',
                'verbose_name': 'file or folder',
            },
        ),
    ]



import os
from setuptools import find_packages, setup
import YaBackup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))

def get_reqs(*fns):
    lst = []
    for fn in fns:
        for package in open(os.path.join(CURRENT_DIR, fn)).readlines():
            package = package.strip()
            if not package:
                continue
            lst.append(package.strip())
    return lst

setup(
    name='django-yabackup',
    version=YaBackup.__version__,
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='A simple Django app to backup your files and database to Yandex.Disk',
    keywords='Yandex.Disk backup',
    long_description=README,
    url='https://github.com/lovelydev/django-yabackup/',
    author='Kirill Tsurkan',
    author_email='kirilltsurkan@@gmail.com',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=get_reqs('requirements.txt'),
    tests_require=get_reqs('requirements.txt'),
)


from django.core.management.base import BaseCommand, CommandError
from YaBackup.backup import YaBackup


class Command(BaseCommand):
    help = 'Run YaBackup'

    def handle(self, *args, **options):
        job = YaBackup()
        job.run_backups()
        self.stdout.write(self.style.SUCCESS('Successfully made backups'))


from django.contrib import admin
from YaBackup.models import Backup, Path
from YaBackup.backup import YaBackup
from django.utils.translation import ugettext_lazy as _


def run_backups_admin(self, request, queryset):
    backup = YaBackup()
    backup.run_backups(queryset, manual=True)
    self.message_user(request, _('Successfully made %s backups!' % queryset.count()))
run_backups_admin.short_description = _('Run backups')


class PathInline(admin.TabularInline):
    model = Path
    extra = 0


@admin.register(Backup)
class BackupAdmin(admin.ModelAdmin):

    list_display = ('title', 'output_directory', 'file_name', 'upload', 'delete_after_upload', 'mysqldump', 'pub_date', )

    fieldsets = (
        (None, {
            'fields': ('title', ('output_directory', 'file_name'), ('upload', 'delete_after_upload'), 'mysqldump')
        }),
        (_('Notes'), {
            'classes': ('collapse',),
            'fields': ('description',),
        }),
    )
    inlines = [
        PathInline,
    ]
    empty_value_display = _('Untitled backup')
    list_filter = ('upload', 'delete_after_upload', 'mysqldump', 'pub_date')
    search_fields = ['title', 'file_name']
    actions = [run_backups_admin]
